@@ -447,7 +447,8 @@ class S3Client {
         $url = $this->buildUrl($bucket, $key) . '?uploads';
         // Use empty payload hash for multipart initiate (required by AWS spec)
         $emptyPayloadHash = 'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855';
-        $headers = $this->buildHeaders('POST', $bucket, $key, 'uploads', 'application/octet-stream', $emptyPayloadHash, 0);
+        // Don't send Content-Type or Content-Length for initiate - R2 might not like them
+        $headers = $this->buildHeadersMinimal('POST', $bucket, $key, 'uploads', $emptyPayloadHash);
 
         // Log the exact URL and headers for debugging
         error_log("Multipart initiate URL: {$url}");
@@ -456,6 +457,7 @@ class S3Client {
         $ch = curl_init();
         curl_setopt($ch, CURLOPT_URL, $url);
         curl_setopt($ch, CURLOPT_POST, true);
+        curl_setopt($ch, CURLOPT_POSTFIELDS, ''); // Explicitly set empty body
         curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
         curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
         curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
@@ -883,6 +885,34 @@ class S3Client {
         return $url;
     }
     
+    private function buildHeadersMinimal($method, $bucket, $key, $queryString = '', $payloadHash = 'UNSIGNED-PAYLOAD') {
+        $timestamp = gmdate('Ymd\THis\Z');
+        $dateStamp = gmdate('Ymd');
+
+        // AWS Signature Version 4
+        $region = $this->config['region'] ?? 'us-east-1';
+        $service = 's3';
+        $credentialScope = $dateStamp . '/' . $region . '/' . $service . '/aws4_request';
+
+        $host = parse_url($this->config['endpoint'], PHP_URL_HOST);
+        $port = parse_url($this->config['endpoint'], PHP_URL_PORT);
+
+        // Include port in host header if it's not standard (80/443)
+        if ($port && $port != 80 && $port != 443) {
+            $host = $host . ':' . $port;
+        }
+
+        $headers = [
+            'Host: ' . $host,
+            'X-Amz-Date: ' . $timestamp,
+            'X-Amz-Content-Sha256: ' . $payloadHash
+        ];
+
+        $headers[] = 'Authorization: ' . $this->signRequestV4Minimal($method, $bucket, $key, $timestamp, $dateStamp, $region, $service, $credentialScope, $queryString, $payloadHash);
+
+        return $headers;
+    }
+
     private function buildHeaders($method, $bucket, $key, $queryString = '', $contentType = 'application/octet-stream', $payloadHash = 'UNSIGNED-PAYLOAD', $contentLength = null) {
         $timestamp = gmdate('Ymd\THis\Z');
         $dateStamp = gmdate('Ymd');
@@ -917,6 +947,67 @@ class S3Client {
         return $headers;
     }
     
+    private function signRequestV4Minimal($method, $bucket, $key, $timestamp, $dateStamp, $region, $service, $credentialScope, $queryString = '', $payloadHash = 'UNSIGNED-PAYLOAD') {
+        // AWS Signature Version 4 - Minimal version for initiate multipart
+        $accessKey = $this->config['access_key'];
+        $secretKey = $this->config['secret_key'];
+
+        // Create canonical URI
+        $parts = [$bucket];
+        if (!empty($key)) {
+            $keyParts = explode('/', $key);
+            foreach ($keyParts as $part) {
+                if ($part !== '') {
+                    $parts[] = rawurlencode($part);
+                }
+            }
+        }
+        $canonicalUri = '/' . implode('/', $parts);
+
+        $canonicalQueryString = $queryString;
+
+        // Create canonical headers - minimal set
+        $host = parse_url($this->config['endpoint'], PHP_URL_HOST);
+        $port = parse_url($this->config['endpoint'], PHP_URL_PORT);
+
+        if ($port && $port != 80 && $port != 443) {
+            $host = $host . ':' . $port;
+        }
+
+        $canonicalHeaders = "host:" . $host . "\n" .
+                           "x-amz-content-sha256:" . $payloadHash . "\n" .
+                           "x-amz-date:" . $timestamp . "\n";
+        $signedHeaders = 'host;x-amz-content-sha256;x-amz-date';
+
+        $canonicalRequest = $method . "\n" .
+                           $canonicalUri . "\n" .
+                           $canonicalQueryString . "\n" .
+                           $canonicalHeaders . "\n" .
+                           $signedHeaders . "\n" .
+                           $payloadHash;
+
+        // Create string to sign
+        $algorithm = 'AWS4-HMAC-SHA256';
+        $stringToSign = $algorithm . "\n" .
+                       $timestamp . "\n" .
+                       $credentialScope . "\n" .
+                       hash('sha256', $canonicalRequest);
+
+        // Create signing key
+        $signingKey = $this->getSigningKey($secretKey, $dateStamp, $region, $service);
+
+        // Create signature
+        $signature = hash_hmac('sha256', $stringToSign, $signingKey);
+
+        // Create authorization header
+        $authorization = $algorithm . ' ' .
+                        'Credential=' . $accessKey . '/' . $credentialScope . ', ' .
+                        'SignedHeaders=' . $signedHeaders . ', ' .
+                        'Signature=' . $signature;
+
+        return $authorization;
+    }
+
     private function signRequestV4($method, $bucket, $key, $timestamp, $dateStamp, $region, $service, $credentialScope, $queryString = '', $contentType = 'application/octet-stream', $payloadHash = 'UNSIGNED-PAYLOAD', $contentLength = null) {
         // AWS Signature Version 4
         $accessKey = $this->config['access_key'];
